import hashlib
import os
import time
import json
import base64
import requests
import datetime
import subprocess

from enum import Enum
from io import BytesIO
from typing import Any
from flask import Request, Response, abort, jsonify
from PIL import Image, UnidentifiedImageError

from chocolate_app.tables import Users, Libraries
from chocolate_app import all_auth_tokens, get_dir_path, LOG_PATH, get_language_file

dir_path = get_dir_path()
LANGUAGE_FILE = get_language_file()


class Codes(Enum):
    CONTINUE_REQUEST = 101
    PROCESSING = 102

    SUCCESS = 201
    MISSING_DATA = 202
    NO_RETURN_DATA = 203
    INVALID_MEDIA_TYPE = 204

    USER_CREATED = 211
    USER_DELETED = 212
    USER_UPDATED = 213
    USER_FOUND = 214
    USER_NOT_FOUND = 215
    USER_ALREADY_EXISTS = 216
    USER_NOT_CREATED = 217
    USER_NOT_DELETED = 218
    USER_NOT_UPDATED = 219

    LIBRARY_CREATED = 221
    LIBRARY_DELETED = 222
    LIBRARY_UPDATED = 223
    LIBRARY_FOUND = 224
    LIBRARY_NOT_FOUND = 225
    LIBRARY_ALREADY_EXISTS = 226
    LIBRARY_NOT_CREATED = 227
    LIBRARY_NOT_DELETED = 228
    LIBRARY_NOT_UPDATED = 229

    SETTINGS_UPDATED = 231
    SETTINGS_NOT_UPDATED = 232

    LOGIN_SUCCESS = 241
    LOGIN_FAILED = 242
    NOT_LOGGED_IN = 243
    LOGGED_IN = 244
    WRONG_PASSWORD = 245
    INVALID_TOKEN = 246

    MEDIA_NOT_FOUND = 251

    INVALID_METHOD = 261


def generate_log(request: Request, component: str) -> None:
    """
    Generate a log

    Args:
        request: The request
        component: The component of the log

    Returns:
        None
    """
    method = request.method

    token = request.headers.get("Authorization")
    ip_address = request.remote_addr
    path = request.path

    try:
        data = request.get_json()
    except Exception:
        data = None

    if token and token in all_auth_tokens:
        user = all_auth_tokens[token]["user"]
        if user:
            try:
                user = Users.query.filter_by(name=user).first()
                if user:
                    username = user.name
                else:
                    username = f"token {token}"
            except Exception:
                username = f"token {token}"
        else:
            username = f"token {token}"
    else:
        username = f"Token {token}"

    if username == "Token Bearer null":
        username = "Unknown"

    if not data:
        message = f"Request {method} at {path}. from {username}"
    else:
        if "password" in data:
            data["password"] = "********"
        if "image" in data:
            data["image"] = "Image as a base64 (to long)"
        message = (
            f"Request {method} at {path}. from {username} with data: {json.dumps(data)}"
        )

    log("INFO", component, message, ip_address)


def log(log_type: str, log_composant=None, log_message="", ip_address="") -> None:
    """
    Log a message

    Args:
        log_type (str): The type of the log
        log_composant (str, optional): The component of the log. Defaults to None.
        log_message (str, optional): The message of the log. Defaults to "".
        ip_address (str, optional): The IP address of the log. Defaults to "".

    Returns:
        None
    """

    the_time = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
    log = f"{the_time} - [{log_type}]"
    if log_composant:
        log += f" [{log_composant}] {log_message}\n"

    if ip_address and ip_address != "":
        log = f"[{ip_address}] {log}"

    # if file does not exist, create it
    if not os.path.exists(LOG_PATH):
        with open(LOG_PATH, "w") as logs:
            logs.write(log)
        return

    with open(LOG_PATH, "r") as logs:
        if log in logs.read():
            return

    with open(LOG_PATH, "a") as logs:
        logs.write(log)


def path_join(*args) -> str:
    """
    Join the path with the correct separator

    Args:
        *args: The path to join

    Returns:
        str: The joined path
    """
    return "/".join(args).replace("\\", "/")


def check_authorization(request, token=None, library=None):
    if not token:
        token = request.headers.get("Authorization")

    if token not in all_auth_tokens:
        generate_log(request, "UNAUTHORIZED")
        abort(401)

    username = all_auth_tokens[token]["user"]

    if library:
        the_lib = Libraries.query.filter_by(name=library).first()

        if not the_lib:
            generate_log(request, "ERROR")
            abort(404)

        user = Users.query.filter_by(name=username).first()
        user_in_the_lib = user_in_lib(user.id, the_lib)

        if not user_in_the_lib:
            generate_log(request, "UNAUTHORIZED")
            abort(401)

        if the_lib is None or user is None:
            generate_log(request, "ERROR")
            abort(404)


def check_admin(request, token):
    if token not in all_auth_tokens:
        generate_log(request, "UNAUTHORIZED")
        abort(401)

    username = all_auth_tokens[token]["user"]

    user = Users.query.filter_by(name=username).first()

    if user is None or not user.account_type == "Admin":
        generate_log(request, "UNAUTHORIZED")
        abort(401)


def user_in_lib(user_id, lib):
    user = Users.query.filter_by(id=user_id).first()

    if not user or not lib:
        return False

    user_id = str(user.id)

    if not isinstance(lib, dict):
        lib = lib.__dict__

    available_for = str(lib["available_for"]).split(",")

    if not lib["available_for"] or user_id in available_for:
        return True
    return False


def save_image(url, path) -> str | None:
    image_requests = requests.Session()
    if not os.path.exists(f"{path}.webp"):
        with open(f"{path}.png", "wb") as f:
            try:
                f.write(image_requests.get(url).content)
            except Exception:
                time.sleep(0.5)
                return save_image(url, path)

        try:
            image = Image.open(f"{path}.png")
        except UnidentifiedImageError:
            return None

        image.save(f"{path}.webp", "webp", optimize=True)
        if os.path.exists(f"{path}.png"):
            os.remove(f"{path}.png")

    return f"{path}.webp"


def check_extension(file, extensions: list) -> bool:
    """
    Check if the file is of a certain type

    Args:
        file (str): The file to check
        extensions (list): The list of extensions to check

    Returns:
        bool: True if the file is of a certain type, False otherwise
    """

    if file.split(".")[-1] in extensions:
        return True
    return False


def is_video_file(file: str) -> bool:
    """
    Check if the file is a video file

    Args:
        file (str): The file to check

    Returns:
        bool: True if the file is a video file, False otherwise
    """
    extensions = ["mkv", "avi", "mp4", "webm", "ogg", "m4v", "mov", "wmv", "flv", "3gp"]
    return check_extension(file, extensions)


def is_music_file(file: str) -> bool:
    """
    Check if the file is a music file

    Args:
        file (str): The file to check

    Returns:
        bool: True if the file is a music file, False otherwise
    """
    extensions = ["mp3", "wav", "ogg", "flac", "m4a", "wma"]
    return check_extension(file, extensions)


def is_book_file(file: str) -> bool:
    """
    Check if the file is a book file

    Args:
        file (str): The file to check

    Returns:
        bool: True if the file is a book file, False otherwise
    """
    extensions = ["pdf", "epub", "cbz", "cbr"]
    return check_extension(file, extensions)


def is_image_file(file: str) -> bool:
    """
    Check if the file is an image file

    Args:
        file (str): The file to check

    Returns:
        bool: True if the file is an image file, False otherwise
    """
    extensions = ["png", "jpg", "jpeg", "gif", "webp"]
    return check_extension(file, extensions)


def is_compressed_file(file: str) -> bool:
    """
    Check if the file is a compressed file

    Args:
        file (str): The file to check

    Returns:
        bool: True if the file is a compressed file, False otherwise
    """
    extensions = ["zip", "rar", "7z", "tar", "gz", "bz2", "xz"]
    return check_extension(file, extensions)


def is_directory(path: str) -> bool:
    """
    Check if the path is a directory

    Args:
        path (str): The path to check

    Returns:
        bool: True if the path is a directory, False otherwise
    """
    return os.path.isdir(path)


def get_chunk_user_token(request: Request) -> int | None:
    """
    Get the user token from the request

    Args:
        request: The request

    Returns:
        str: The user token
    """
    token = request.headers.get("Authorization")
    if token not in all_auth_tokens:
        return None
    user = all_auth_tokens[token]["user"]
    return Users.query.filter_by(name=user).first().id


def generate_response(code: Codes, error=False, data: Any = None) -> Response:
    """
    Function to generate a response, based on a model
    """

    output_json = {
        "code": code.value,
        "error": error,
        "message": Codes(code).name,
        "data": data,
    }

    return jsonify(output_json)


def generate_b64_image(
    image_path: str | None, width: int | None = None, height: int | None = None
) -> str | None:
    """
    Generate a base64 image

    Args:
        image_path (str | None): The path of the image

    Returns:
        str: The base64 image
    """
    if not image_path:
        return None

    if not os.path.exists(image_path):
        return None

    with open(image_path, "rb") as image_file:
        image_b64 = base64.b64encode(image_file.read()).decode("utf-8")

    image = Image.open(BytesIO(base64.b64decode(image_b64)))
    if width:
        wpercent = width / float(image.size[0])
        hsize = int((float(image.size[1]) * float(wpercent)))
        image = image.resize((width, hsize), Image.ANTIALIAS)
    if height:
        hpercent = height / float(image.size[1])
        wsize = int((float(image.size[0]) * float(hpercent)))
        image = image.resize((wsize, height), Image.ANTIALIAS)

    img_io = BytesIO()
    image.save(img_io, "WEBP")
    img_io.seek(0)
    img_str = base64.b64encode(img_io.getvalue()).decode("utf-8")

    return f"data:image/png;base64,{img_str}"


def translate(query: str | int) -> str:
    """
    Translate a query

    Args:
        query (str | number): The query to get the translation

    Returns:
        str: The translated query
    """
    if not query:
        return ""

    if isinstance(query, int):
        query = str(query)

    return LANGUAGE_FILE[query]


def hash_string(string: str) -> str:
    return hashlib.md5(string.encode()).hexdigest()


def length_video(path: str) -> float:
    seconds = subprocess.run(
        [
            "ffprobe",
            "-v",
            "error",
            "-show_entries",
            "format=duration",
            "-of",
            "default=noprint_wrappers=1:nokey=1",
            path,
        ],
        stdout=subprocess.PIPE,
        text=True,
    )
    try:
        return float(seconds.stdout)
    except ValueError:
        return 0.0



import os

from iso639 import Lang

from chocolate_app import DB, get_config, get_dir_path, write_config
from chocolate_app.routes.api.auth import token_required
from flask import Blueprint, request, Response
from chocolate_app.tables import Libraries, Users
from chocolate_app.utils.utils import generate_response, Codes


settings_bp = Blueprint("settings", __name__, url_prefix="/settings")


def clean_json_for_config(json: dict) -> dict:
    for key, value in json.items():
        if isinstance(value, dict):
            json[key] = clean_json_for_config(value)
        elif value is True:
            json[key] = "true"
        elif value is False:
            json[key] = "false"
        elif value.isdigit():
            json[key] = int(value)
        elif value.replace(".", "", 1).isdigit():
            json[key] = float(value)
    return json


def handle_general_settings(method) -> Response:
    def get() -> Response:
        config = get_config()
        return generate_response(Codes.SUCCESS, data=config.__dict__["_sections"])

    def post() -> Response:
        if not request.json:
            return generate_response(Codes.INVALID_MEDIA_TYPE)

        data = request.json

        cleaned_data = clean_json_for_config(data)

        write_config(cleaned_data)

        return generate_response(Codes.SUCCESS)

    if method == "GET":
        return get()
    elif method == "POST":
        return post()
    else:
        return generate_response(Codes.INVALID_METHOD)


def handle_accounts_settings(method) -> Response:
    def get() -> Response:
        accounts = Users.query.all()
        accounts_list = []
        for account in accounts:
            accounts_list.append(
                {
                    "id": account.id,
                    "name": account.name,
                    "account_type": account.account_type,
                }
            )

        return generate_response(Codes.SUCCESS, data=accounts_list)

    def post() -> Response:
        if not request.json:
            return generate_response(Codes.INVALID_MEDIA_TYPE)

        data = request.json

        if "id" not in data:
            return generate_response(Codes.MISSING_DATA, True)

        account = Users.query.filter_by(id=data["id"]).first()

        if not account:
            return generate_response(Codes.USER_NOT_FOUND, True)

        if "name" in data:
            account.name = data["name"]

        if "account_type" in data:
            account.account_type = data["account_type"]

        DB.session.commit()

        return generate_response(Codes.SUCCESS)

    def delete() -> Response:
        if not request.json:
            return generate_response(Codes.INVALID_MEDIA_TYPE)

        data = request.json

        if "id" not in data:
            return generate_response(Codes.MISSING_DATA, True)

        account = Users.query.filter_by(id=data["id"]).first()

        if not account:
            return generate_response(Codes.USER_NOT_FOUND, True)

        DB.session.delete(account)
        DB.session.commit()

        return generate_response(Codes.SUCCESS)

    def put() -> Response:
        if not request.json:
            return generate_response(Codes.INVALID_MEDIA_TYPE)

        data = request.json

        if "name" not in data or "password" not in data or "account_type" not in data:
            return generate_response(Codes.MISSING_DATA, True)

        user = Users(
            name=data["name"],
            password=data["password"],
            account_type=data["account_type"],
            profile_picture="static/images/default_profile_picture.jpg",
        )
        DB.session.add(user)
        DB.session.commit()

        return generate_response(Codes.SUCCESS)

    if method == "GET":
        return get()
    elif method == "POST":
        return post()
    elif method == "DELETE":
        return delete()
    elif method == "PUT":
        return put()
    else:
        return generate_response(Codes.INVALID_METHOD)


def handle_libraries_settings(method) -> Response:
    def get() -> Response:
        libraries = Libraries.query.all()
        libraries_list = []
        for library in libraries:
            libraries_list.append(
                {
                    "id": library.id,
                    "name": library.name,
                    "path": library.folder,
                    "type": library.type,
                }
            )

        return generate_response(Codes.SUCCESS, data=libraries_list)

    def post() -> Response:
        if not request.json:
            return generate_response(Codes.INVALID_MEDIA_TYPE)

        data = request.json

        if "id" not in data:
            return generate_response(Codes.MISSING_DATA, True)

        library = DB.session.query(Libraries).filter_by(id=data["id"]).first()

        if "scan" in data and data["scan"]:
            # TODO: Implement the scan function
            return generate_response(Codes.SUCCESS)

        if not library:
            return generate_response(Codes.LIBRARY_NOT_FOUND, True)

        if "name" in data:
            library.name = data["name"]

        if "path" in data:
            library.folder = data["path"]

        DB.session.commit()

        return generate_response(Codes.SUCCESS)

    def delete() -> Response:
        if not request.json:
            return generate_response(Codes.INVALID_MEDIA_TYPE)

        data = request.json

        if "id" not in data:
            return generate_response(Codes.MISSING_DATA, True)

        library = DB.session.query(Libraries).filter_by(id=data["id"]).first()

        if not library:
            return generate_response(Codes.LIBRARY_NOT_FOUND, True)

        DB.session.delete(library)
        DB.session.commit()

        return generate_response(Codes.SUCCESS)

    def put() -> Response:
        if not request.json:
            return generate_response(Codes.INVALID_MEDIA_TYPE)

        data = request.json

        if "name" not in data or "path" not in data:
            return generate_response(Codes.MISSING_DATA, True)

        library = Libraries(
            name=data["name"],
            folder=data["path"],
            type=data["type"],
            image="",
            available_for=None,
        )

        DB.session.add(library)
        DB.session.commit()

        return generate_response(Codes.SUCCESS)

    if method == "GET":
        return get()
    elif method == "POST":
        return post()
    elif method == "DELETE":
        return delete()
    elif method == "PUT":
        return put()
    else:
        return generate_response(Codes.INVALID_METHOD)


@settings_bp.route("/<section>", methods=["GET", "POST", "PUT", "DELETE"])
@token_required
def settings(section) -> Response:
    if section == "general":
        return handle_general_settings(request.method)
    elif section == "accounts":
        return handle_accounts_settings(request.method)
    elif section == "libraries":
        return handle_libraries_settings(request.method)
    else:
        return generate_response(Codes.MISSING_DATA, True)


@settings_bp.route("/languages", methods=["GET"])
def languages() -> Response:
    languages = []
    dir_path = get_dir_path()
    for file in os.listdir(f"{dir_path}/static/lang"):
        if file.endswith(".json"):
            languages.append(file[:-5])

    languages_list = []
    for language in languages:
        languages_list.append(
            {
                "code": language,
                "name": Lang(language).name,
            }
        )

    languages_list = sorted(languages_list, key=lambda x: x["name"])

    return generate_response(Codes.SUCCESS, data=languages_list)



# Copyright (C) 2024 Impre_visible
import warnings
import requests
import threading
import sqlalchemy

from sqlalchemy import inspect
from sqlalchemy.sql import text
from watchdog.observers import Observer
from time import sleep
from watchdog.events import FileSystemEventHandler

from flask import (
    jsonify,
    Response,
    request,
    send_file,
    render_template,
)

from chocolate_app import (
    app,
    SERVER_PORT,
    get_dir_path,
    DB,
    LOGIN_MANAGER,
    tmdb,
    config,
    ARGUMENTS,
    scans,
    get_language_file,
)
from chocolate_app.tables import (
    Movies,
    Episodes,
    OthersVideos,
    Users,
    Libraries,
)
from chocolate_app.utils.utils import generate_log
from chocolate_app.plugins_loader import events, routes

dir_path: str = get_dir_path()

with app.app_context():
    DB.create_all()


with warnings.catch_warnings():
    warnings.simplefilter("ignore", category=sqlalchemy.exc.SAWarning)


@LOGIN_MANAGER.user_loader
def load_user(id: int) -> Users | None:
    """
    Load the user from the database

    Args:
        id (int): The user id

    Returns:
        Users | None: The user or None
    """
    return Users.query.get(int(id))


tmdb.language = config["ChocolateSettings"]["language"].lower()
tmdb.debug = True


@app.after_request
def after_request(response: Response) -> Response:
    """
    The after request function

    Args:
        response (Response): The response

    Returns:
        Response: The response
    """

    ip_address = request.remote_addr

    if "CF-Connecting-IP" in request.headers:
        ip_address = request.headers["CF-Connecting-IP"]
    elif "X-Real-IP" in request.headers:
        ip_address = request.headers["X-Real-IP"]
    elif "X-Forwarded-For" in request.headers:
        ip_address = request.headers["X-Forwarded-For"]

    request.remote_addr = ip_address

    generate_log(
        request,
        f"{response.status_code} - {request.method} - {request.path} - {ip_address}",
    )

    return response


@app.route(
    "/",
    methods=[
        "GET",
        "POST",
        "PUT",
        "DELETE",
        "PATCH",
        "HEAD",
        "CONNECT",
        "OPTIONS",
        "TRACE",
    ],
)
@app.route(
    "/<path:path>",
    methods=[
        "GET",
        "POST",
        "PUT",
        "DELETE",
        "PATCH",
        "HEAD",
        "CONNECT",
        "OPTIONS",
        "TRACE",
    ],
)
def index(path=None) -> str | Response:
    """
    The index route

    Args:
        path (str, optional): The path. Defaults to None.

    Returns:
        str: The rendered template
    """

    if path and routes.have_static_file(path):
        return routes.get_static_file(path)

    if path and routes.have_route(path):
        return routes.execute_route(path, request)

    return render_template("index.html")


@app.route("/language_file")
def route_language_file() -> Response:
    return jsonify(get_language_file())


@app.route("/download_other/<video_hash>")
def download_other(video_hash: str) -> Response:
    video = OthersVideos.query.filter_by(video_hash=video_hash).first()
    video = video.__dict__
    del video["_sa_instance_state"]
    return send_file(video["slug"], as_attachment=True)


def is_valid_url(url: str) -> bool:
    try:
        response = requests.get(url)
        return response.status_code == requests.codes.ok
    except requests.exceptions.RequestException:
        return False


@app.route("/is_chocolate", methods=["GET", "POST"])
def is_chocolate() -> Response:
    return jsonify({"is_chocolate": True})


@app.route("/download_movie/<movie_id>")
def download_movie(movie_id: int) -> Response:
    can_download = config["ChocolateSettings"]["allowDownload"].lower() == "true"
    if not can_download:
        return jsonify({"error": "download not allowed"})
    movie = Movies.query.filter_by(id=movie_id).first()
    return send_file(movie.slug, as_attachment=True)


@app.route("/download_episode/<episode_id>")
def download_episode(episode_id: int) -> Response:
    can_download = config["ChocolateSettings"]["allowDownload"].lower() == "true"
    if not can_download:
        return jsonify({"error": "download not allowed"})
    episode = Episodes.query.filter_by(episode_id=episode_id).first()
    episode_path = episode.slug
    return send_file(episode_path, as_attachment=True)


def start_scanning_threads(app) -> None:
    def scan(library: Libraries = None) -> None:
        if library is None:
            return
        with app.app_context():
            update_db_columns(DB.engine, DB)
            library = library.__dict__

            type_to_call = {
                "series": scans.getSeries,
                "consoles": scans.getGames,
                "others": scans.getOthersVideos,
                "books": scans.getBooks,
                "musics": scans.getMusics,
            }

            scanner = None
            if library["type"] == "movies":
                scanner = scans.MovieScanner()
                scanner.set_library_name(library["name"])
                scanner.scan()
            elif library["type"] == "tv":
                scanner = scans.LiveTVScanner()
                scanner.set_library_name(library["name"])
                scanner.scan()
            else:
                scanner = type_to_call[library["type"]]
                scanner(library["name"])

    class ScanEventHandler(FileSystemEventHandler):

        def __init__(self):
            self.timer = None

        def on_any_event(self, event):
            with app.app_context():
                library = (
                    Libraries.query.filter_by(folder=event.src_path).first()
                    or Libraries.query.filter_by(folder=event.src_path + "/").first()
                )
                if event.event_type not in ["created", "deleted"]:
                    return

                if library is None:
                    return

                print(f"Detected event {event.event_type} in {event.src_path}")

                if self.timer is not None:
                    self.timer.cancel()

                self.timer = threading.Timer(30, scan, args=(library,))
                self.timer.start()

    # Launch the scan thread every 24 hours

    with app.app_context():
        # Setup watchdog for directory monitoring
        def start_watchdog(path_to_watch):
            event_handler = ScanEventHandler()
            observer = Observer()
            observer.schedule(event_handler, path=path_to_watch, recursive=True)
            observer.start()
            try:
                while True:
                    sleep(1)
            except KeyboardInterrupt:
                observer.stop()
            observer.join()

        # Launch watchdog thread for the directory to monitor
        for library in Libraries.query.all():
            if library.type == "tv":
                continue
                scanner = scans.LiveTVScanner()
                scanner.set_library_name(library.name)
                scanner.scan()
                continue

            scan(library)

            watchdog_thread = threading.Thread(
                target=start_watchdog, args=(library.folder,)
            )
            watchdog_thread.daemon = True
            watchdog_thread.start()

    print("Scanning threads and watchdog initialized.")


def start_chocolate() -> None:
    events.execute_event(events.Events.BEFORE_START)
    if not ARGUMENTS.no_scans and config["APIKeys"]["TMDB"] != "Empty":
        start_scanning_threads(app)

    app.run(host="0.0.0.0", port=SERVER_PORT)
    events.execute_event(events.Events.AFTER_START)


def update_db_columns(engine, db):
    # Connecter à la base de données
    connection = engine.connect()
    inspector = inspect(engine)

    # Parcourir toutes les classes définies dans Base
    for cls in db.metadata.tables.values():
        # Si la classe est une sous-classe de db.Model
        # print the attributes of the class
        name = None

        if hasattr(cls, "name"):
            name = cls.name
        elif hasattr(cls, "full_name"):
            name = cls.full_name

        if name:
            table_name = name

            columns = inspector.get_columns(table_name)

            class_columns = cls.columns.keys()

            missing_columns = [
                column
                for column in class_columns
                if column not in [col["name"] for col in columns]
            ]

            extra_columns = [
                column
                for column in columns
                if (column["name"] or column["full_name"]) not in class_columns
            ]

            for column in missing_columns:
                column_type = cls.columns[column].type
                connection.execute(
                    text(f"ALTER TABLE {table_name} ADD COLUMN {column} {column_type};")
                )

            for column in extra_columns:
                name = None

                if hasattr(column, "name"):
                    name = column.name
                elif hasattr(column, "full_name"):
                    name = column.full_name
                else:
                    continue

                connection.execute(
                    text(f"ALTER TABLE {table_name} DROP COLUMN {column['name']};")
                )

    connection.close()


if __name__ == "__main__":
    start_chocolate()



import math
import pandas as pd


def overlap(interval1, interval2):
    """
    Returns the total amount of overlap between two intervals in the format of (x,y)

    Example:
        input:      (0,10) , (5,10)
        returns:    5
    """
    return max(0, min(interval1[1], interval2[1]) - max(interval1[0], interval2[0]))


def sum_timestamps(timestamps):
    """
    Get the total number of seconds out of a list with timestamps formatted like: (start,end)
    """
    result = 0
    for start, end in timestamps:
        result += end - start

    return result


# matches two lists of (starttime,endtime) detections and outputs the relevancy variables
def match_detections_precision_recall(detected, ground_truth, verbose=False):
    """
    Compares the detections and ground truth lists of timestamps and calculates
    and outputs precision and recall scores based on the comparison.
    """

    total_relevant_seconds = sum_timestamps(ground_truth)
    total_detected_seconds = sum_timestamps(detected)
    relevant_detected_seconds = 0

    for start, end in ground_truth:
        for i, (start_d, end_d) in enumerate(detected):

            if abs(start - start_d) < 2:
                start_d = start
            if abs(end - end_d) < 2:
                end_d = end

            relevant = overlap((start, end), (start_d, end_d))
            relevant_detected_seconds += relevant

    return total_relevant_seconds, total_detected_seconds, relevant_detected_seconds


def merge_consecutive_timestamps(timestamps):
    """
    Merges consecutive timestamps in a list if they're less than 2 seconds apart
    Example: [(0,5), (5,10), (20,30)] gets combined into [(0,10),[20,30]
    """
    result = []
    i = 0
    while i < len(timestamps):
        (start, end) = timestamps[i]

        # check if we're not already at the last element
        if i < len(timestamps) - 1:
            (start_next, end_next) = timestamps[i + 1]
            # merge if less than 2 seconds apart
            if abs(end - start_next) < 2:
                result.append((start, end_next))
                i += 1
            else:
                result.append((start, end))

        else:
            result.append((start, end))

        i += 1

    return result


def to_seconds(time):
    """
    Converts string of format hh:mm:ss to total number of seconds
    """
    if time == "None":
        return -1
    try:
        hours = int(time.split(":")[0])
        minutes = int(time.split(":")[1])
        seconds = int(float(time.split(":")[2]))
        return hours * 60 * 60 + minutes * 60 + seconds
    except:
        if math.isnan(time):
            return -1


def get_skippable_timestamps_by_filename(filename, df):
    """
    Looks in the supplied pandas dataframe to extract the annotations for the
    given filename.
    """
    result = []
    try:
        row = df.loc[df["filename"] == filename].to_dict(orient="records")[0]

        if not row["recap_start"] == -1:
            result.append((row["recap_start"], row["recap_end"]))
        if not row["openingcredits_start"] == -1:
            result.append((row["openingcredits_start"], row["openingcredits_end"]))
        if not row["preview_start"] == -1:
            result.append((row["preview_start"], row["preview_end"]))
        if not row["closingcredits_start"] == -1:
            result.append((row["closingcredits_start"], row["closingcredits_end"]))
    except Exception as e:
        raise Exception(
            f"Something went wrong when getting the annotations for: {filename} \n Make sure the annotations are there in the correct format"
        )

    return merge_consecutive_timestamps(result)


def get_annotations(filename):
    """
    Gets the annotations file as a pandas dataframe.
    """
    annotations = pd.read_csv(filename).dropna(how="all")

    # convert the timestamp strings to regular seconds
    annotations["recap_start"] = annotations["recap_start"].apply(to_seconds)
    annotations["recap_end"] = annotations["recap_end"].apply(to_seconds)
    annotations["openingcredits_end"] = annotations["openingcredits_end"].apply(
        to_seconds
    )
    annotations["openingcredits_start"] = annotations["openingcredits_start"].apply(
        to_seconds
    )
    annotations["preview_start"] = annotations["preview_start"].apply(to_seconds)
    annotations["preview_end"] = annotations["preview_end"].apply(to_seconds)
    annotations["closingcredits_end"] = annotations["closingcredits_end"].apply(
        to_seconds
    )
    annotations["closingcredits_start"] = annotations["closingcredits_start"].apply(
        to_seconds
    )

    return annotations



from flask import Blueprint

from chocolate_app.routes.api.auth import auth_bp
from chocolate_app.routes.api.profil import profil_bp
from chocolate_app.routes.api.medias import medias_bp
from chocolate_app.routes.api.watch import watch_bp
from chocolate_app.routes.api.settings import settings_bp

api_bp = Blueprint('api', __name__, url_prefix='/api')

api_bp.register_blueprint(auth_bp)
api_bp.register_blueprint(medias_bp)
api_bp.register_blueprint(watch_bp)
api_bp.register_blueprint(settings_bp)
api_bp.register_blueprint(profil_bp)
